@@ -1,5 +1,7 @@
 library("bigmemory")
 context("morder/mpermute")
+data(iris,package='datasets')
+
 
 m = matrix(as.double(as.matrix(iris[,1:4])), nrow=nrow(iris[,1:4]))
 n = m
@@ -21,7 +23,9 @@ test_that("mpermute changes elements order",{
 test_that("column reording works", {
   mpermuteCols(bm, order = c(3,4,1,2))
   expect_equivalent(bm[], mm[,c('c','d','a','b')])
+  expect_equivalent(colnames(bm), c('c','d','a','b'))
   mpermuteCols(mm, order = c(3,4,1,2))
+  expect_equivalent(colnames(mm), c('c','d','a','b'))
   expect_equivalent(bm[], mm)
   mpermuteCols(bm, rows = 1)
   mpermuteCols(mm, rows = 1)
